/*------------------------------------------------------*
 | Author: Maurizio Loreti, aka MLO or (HAM) I3NOO      |
 | Work:   University of Padova - Department of Physics |
 |         Via F. Marzolo, 8 - 35131 PADOVA - Italy     |
 | Phone:  +39 (049) 827-7216   FAX: +39 (049) 827-7102 |
 | EMail:  loreti@pd.infn.it                            |
 | WWW:    http://www.pd.infn.it/~loreti/mlo.html       |
 *------------------------------------------------------*

 Straight implementation of heapsort; the code is taken
 from Sedgewick - "Algorithms in C" (3rd Ed) - Addison
 Wesley.  The procedures have been compacted and the
 array in fixDown starts from element 0 to avoid
 Sedgewick's dirty hack.

 $Id: heapsort.c,v 1.1 2000/02/07 17:36:51 loreti Exp $

 *------------------------------------------------------*/

#include <string.h>

/**
 | The following entries specialize this procedure for the
 | sorting of an array of integers in ascending order
**/

typedef int Item;               /* Objects to be sorted */
#define less(i,j) (*(i) < *(j)) /* Comparison method    */

static void fixDown(Item *, size_t, const size_t);
static void swap(Item *, Item *);

void sort(
  Item         *array,          /* Array to be sorted */
  const size_t  N               /* Number of elements */
) {
  size_t k = N/2;

  while (k > 0) fixDown(array, k--, N);

  for (k = N-1;   k > 0;   k--) {
    swap(array, array+k);
    fixDown(array, 1, k);
  }
}

static void fixDown(
  Item         *array,          /* Heap address */
  size_t        k,              /* Starting element for top-down fix-up */
  const size_t  N               /* Heap size */
) {
  size_t j;

  for (j = 2*k;   j <= N;   k = ++j,  j = 2*k) {
    if (j < N   &&   less(array+j-1, array+j)) j++;
    if (less(array+(--j), array+(--k))) {
      break;
    } else {
      swap(array+j, array+k);
    }
  }
}

static void swap(
  Item *first,
  Item *second
) {
  Item temp = *first;
  *first = *second;
  *second = temp;
}
